/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

/**
 * @file 	ALDOpRunner.cc
 * @brief	Contains the command line operator runner application.
 */

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <map>
#include <string>

#include "dataio/ALDDataIOManagerCmdline.h"
#include "operator/ALDOperatorManager.h"
#include "operator/ALDOperator.h"

using namespace std;

using namespace Alida;

/// Main function to call the commandline operator runner.
/**
 * The operator runner executes operators as specified via commandline.
 * Its usage is as follows:
 *
 * AlidaOpRunner \<operator\> [param=value]
 *
 * @param argc		Contains number of arguments upon invocation.
 * @param	argv		Contains list of commandline arguments upon invocation.
 * @return Exit status of the application, unequal to 0 in case of error.
 */
int main( int argc, const char* argv[] )
{
  printf( "\nAlidaOpRunner is running...\n\n" );

  if (argc < 2) {
  	cout << "Please choose an operator!\n" << endl;
  	ALDOperatorManager::Instance().printOperatorHash();
  	exit(-1);
  }

  // get operator object
  ALDOperator *op = ALDOperatorManager::Instance().getOperator(argv[1]);
  if (op == NULL) {
  	cout << "Unknown operator \"" << argv[1] << "\"... check the spelling!"
  				<< endl;
  	exit(-1);
  }

  // list parameters
  cout << "Chosen operator : " << argv[1] << endl;

  // get list of parameters
  map<string, ALDOpParameterDescriptor*> params =	op->getParameters();

  // print parameters if none is provided
  if (argc==2) {
  	map<string, ALDOpParameterDescriptor*>::const_iterator it;
  	string directionName;
  	cout << "Operator parameters: " << endl;
  	for (it = params.begin(); it != params.end(); ++it) {
  		switch(it->second->direction)
  		{
  		case PARAMETER_IN:
  			directionName = "IN";
  			break;
  		case PARAMETER_OUT:
  			directionName = "OUT";
  			break;
  		case PARAMETER_INOUT:
  			directionName = "INOUT";
  			break;
  		}
  		cout 	<< " Name = " << it->first
  					<< " , type = " << it->second->typeNiceName
  					<< " , explanation = " << it->second->explanation
  					<< " , direction = " << directionName << std::endl;
  	}
  	cout << endl;
  	cout << "Please specify appropriate values!" << endl << endl;
  	exit(0);
  }

  // parse input parameters
  cout << "Configuration: " << endl;
  for (int i = 2; i < argc; ++i) {
  	// convert argument to string
  	string argument(argv[i]);
  	unsigned int equalSignPos = argument.find('=');
  	if (equalSignPos == string::npos) {
    	cout 	<< i << " th parameter argument (" << argv[i] << ")"
    				<< " - error in argument string, skipping!" << endl;
    	continue;
  	}
  	// get parameter name and value string
  	string param= argument.substr(0,equalSignPos);
  	string value= argument.substr(equalSignPos+1);

  	// savety check if parameter is known
  	if (params[param] == NULL) {
  		cout << "Parameter unknown, skipping \"" << param << "\"..." << endl;
  		continue;
  	}

  	// check if the parameter is really an input parameter
  	if (params[param]->direction == PARAMETER_OUT)
  		continue;

  	// read parameter value
  	void *paramVal=
  			ALDDataIOManagerCmdline::Instance().readData(value,params[param]->typeIdStr);
  	if (paramVal == NULL) {
  		cout << endl << "Error reading parameters... exiting!" << endl;
  		exit(-1);
  	}
  	// set parameter
  	op->setParameter(param, paramVal);
  	cout << "Parameter \"" << param << "\" set.." << endl;
  }

  // run the operator
  cout << endl << "AlidaOpRunner - running the operator..." << endl;
  try {
      op->runOp();
  } catch (ALDException ex) {
      ex.printMsg();
      exit(-1);
  }
  cout << endl << "AlidaOpRunner - finished operator execution!" << endl;
  
  // display/save the result data
  cout << endl << "Displaying / saving results..." << endl << endl;
  for (int i = 2; i < argc; ++i) {
  	// convert argument to string
  	string argument(argv[i]);
  	unsigned int equalSignPos = argument.find('=');
  	if (equalSignPos == string::npos) {
    	cout 	<< i << " th parameter argument (" << argv[i] << ")"
    				<< " - error in argument string, skipping!" << endl;
    	continue;
  	}
  	// get parameter name and value string
  	string param= argument.substr(0,equalSignPos);
  	string value= argument.substr(equalSignPos+1);

  	// savety check if parameter is known
  	if (params[param] == NULL) {
  		cout << "Parameter unknown, skipping \"" << param << "\"..." << endl;
  		continue;
  	}

  	// check if the parameter is really an input parameter
  	if (params[param]->direction == PARAMETER_IN)
  		continue;

  	// write output parameter result value
  	string resultval = ALDDataIOManagerCmdline::Instance().writeData(
  								value,params[param]->typeIdStr, op->getParameter(param));
  	if (!resultval.empty())
  		cout << "Parameter \"" << param << "\" - value = " << resultval << endl;
  	else
  		cout << "Parameter \"" << param << "\" - <internal handling>" << endl;
  }
  printf("\n... AlidaOpRunner finished completely!\n\n");
}

